"""
Project: Azure Kinect management
Author: Juan Carlos Miranda
Date: May 2022
Description:
Example of testing Azure Kinect functions to generates 3D point clouds

Usage:
python -m unittest $HOME/development/ak_sm_recorded/src/camera_classes/test_job_thread_3d_azure.py

"""
import unittest
import os
import time
import helpers.helper_load_config as hc
from camera_classes.job_thread_3d_azure import JobThread3DAzure


class ServiceExit(Exception):
    """
    Custom exception which is used to trigger the clean exit
    of all running threads and the main program.
    """
    print('Service Exit --> \r')
    pass


def service_shutdown(signum, frame):
    print('Service shut down --> Caught signal %d' % signum)
    raise ServiceExit


class TestJobThread3DAzure(unittest.TestCase):
    def setUp(self):
        self.root_folder = os.path.abspath('')
        self.path_conf_file = os.path.join(self.root_folder, 'test_conf', 'test_kinect_azure_settings.conf')
        self.path_video_output = os.path.join(self.root_folder, 'test_recorded_video', )
        self.my_device_configuration = hc.load_config_from_file(self.path_conf_file)
        pass

    def test_launch_thread_3d_azure(self):
        # ----------------
        try:
            print('Running functions from threads --> ')
            job_ak_camera_3d = JobThread3DAzure(self.my_device_configuration, self.path_video_output)
            job_ak_camera_3d.start()
            print('Waiting 5 seconds and recording data--> ')
            time.sleep(4)
            job_ak_camera_3d.shutdown_flag.set()
            job_ak_camera_3d.join()
            print('Video saved in --> ', self.path_video_output)
            flag_exist_video_list = os.listdir(self.path_video_output)
            if flag_exist_video_list is not None:
                flag_video_created = True
            # cleaning all .mkv files
            for a_filename in os.listdir(self.path_video_output):
                if a_filename.endswith('.xyz'):
                    print(a_filename)
                    os.remove(os.path.join(self.path_video_output, a_filename))
        # ----------------
        except ServiceExit:
            print('ServiceExit Exception --> ')
            print('CATCHING any Exception HERE!-->')
            job_ak_camera_3d.shutdown_flag.set()
            job_ak_camera_3d.join()
        # ----------------
        self.assertEqual(True, flag_video_created)



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: August 2021
Description:

Use:
"""

import configparser
from pyk4a import Config, ColorResolution, ImageFormat, DepthMode
from pyk4a import FPS, WiredSyncMode


def convert_color_resolution(convert_param):
    color_resolution = ColorResolution.RES_720P
    if convert_param == 'OFF':
        color_resolution = ColorResolution.OFF
    elif convert_param == 'RES_720P':
        color_resolution = ColorResolution.RES_720P
    elif convert_param == 'RES_1080P':
        color_resolution = ColorResolution.RES_1080P
    elif convert_param == 'RES_1440P':
        color_resolution = ColorResolution.RES_1440P
    elif convert_param == 'RES_1536P':
        color_resolution = ColorResolution.RES_1536P
    elif convert_param == 'RES_2160P':
        color_resolution = ColorResolution.RES_2160P
    elif convert_param == 'RES_3072P':
        color_resolution = ColorResolution.RES_3072P
    return color_resolution

def convert_color_format(convert_param):
    color_format = ImageFormat.COLOR_BGRA32
    if convert_param == 'COLOR_MJPG':
        color_format = ImageFormat.COLOR_MJPG
    elif convert_param == 'COLOR_NV12':
        color_format = ImageFormat.COLOR_NV12
    elif convert_param == 'COLOR_YUY2':
        color_format = ImageFormat.COLOR_YUY2
    elif convert_param == 'COLOR_BGRA32':
        color_format = ImageFormat.COLOR_BGRA32
    elif convert_param == 'DEPTH16':
        color_format = ImageFormat.DEPTH16
    elif convert_param == 'IR16':
        color_format = ImageFormat.IR16
    elif convert_param == 'CUSTOM8':
        color_format = ImageFormat.CUSTOM8
    elif convert_param == 'CUSTOM16':
        color_format = ImageFormat.CUSTOM16
    elif convert_param == 'CUSTOM':
        color_format = ImageFormat.CUSTOM
    return color_format


def convert_depth_mode(convert_param):
    depth_mode = DepthMode.NFOV_UNBINNED
    if convert_param == 'OFF':
        depth_mode = DepthMode.OFF
    elif convert_param == 'NFOV_2X2BINNED':
        depth_mode = DepthMode.NFOV_2X2BINNED
    elif convert_param == 'NFOV_UNBINNED':
        depth_mode = DepthMode.NFOV_UNBINNED
    elif convert_param == 'WFOV_2X2BINNED':
        depth_mode = DepthMode.WFOV_2X2BINNED
    elif convert_param == 'WFOV_UNBINNED':
        depth_mode = DepthMode.WFOV_UNBINNED
    elif convert_param == 'PASSIVE_IR':
        depth_mode = DepthMode.PASSIVE_IR
    return depth_mode

def convert_camera_fps(convert_param):
    camera_fps = FPS.FPS_30
    if convert_param == 'FPS_5':
        camera_fps = FPS.FPS_5
    elif convert_param == 'FPS_15':
        camera_fps = FPS.FPS_15
    elif convert_param == 'FPS_30':
        camera_fps = FPS.FPS_30
    return camera_fps

def convert_wired_sync_mode(convert_param):
    wired_sync_mode = WiredSyncMode.STANDALONE
    if convert_param == 'MASTER':
        wired_sync_mode = WiredSyncMode.MASTER
    elif convert_param == 'SUBORDINATE':
        wired_sync_mode = WiredSyncMode.SUBORDINATE
    elif convert_param == 'STANDALONE':
        wired_sync_mode = WiredSyncMode.STANDALONE
    return wired_sync_mode


def load_config_from_file(f_config_name: object) -> object:
    """
    Read config from file settings.conf
    :return:
    """
    f_config = configparser.ConfigParser()
    f_config.read(f_config_name)
    dev_conf = Config()
    dev_conf.color_resolution = convert_color_resolution(f_config['DEFAULT']['color_resolution'])
    dev_conf.color_format = convert_color_format(f_config['DEFAULT']['color_format'])
    dev_conf.depth_mode = convert_depth_mode(f_config['DEFAULT']['depth_mode'])
    dev_conf.camera_fps = convert_camera_fps(f_config['DEFAULT']['camera_fps'])
    dev_conf.synchronized_images_only = bool(f_config['DEFAULT']['synchronized_images_only'])
    dev_conf.depth_delay_off_color_usec = int(f_config['DEFAULT']['depth_delay_off_color_usec'])
    dev_conf.wired_sync_mode = convert_wired_sync_mode(f_config['DEFAULT']['wired_sync_mode'])
    dev_conf.subordinate_delay_off_master_usec = int(f_config['DEFAULT']['subordinate_delay_off_master_usec'])
    dev_conf.disable_streaming_indicator = bool(f_config['DEFAULT']['disable_streaming_indicator'])
    return dev_conf


def save_config_in_file(f_config_name: object, dev_conf: Config) -> object:
    '''
    Save config in file settings.conf
    :return:
    '''
    f_config = configparser.ConfigParser()

    f_config['DEFAULT']['color_resolution'] = dev_conf.color_resolution.name
    f_config['DEFAULT']['color_format'] = dev_conf.color_format.name
    f_config['DEFAULT']['depth_mode'] = dev_conf.depth_mode.name
    f_config['DEFAULT']['camera_fps'] = dev_conf.camera_fps.name
    f_config['DEFAULT']['synchronized_images_only'] = dev_conf.synchronized_images_only.__str__()
    f_config['DEFAULT']['depth_delay_off_color_usec'] = dev_conf.depth_delay_off_color_usec.__str__()
    f_config['DEFAULT']['wired_sync_mode'] = dev_conf.wired_sync_mode.name
    f_config['DEFAULT']['subordinate_delay_off_master_usec'] = dev_conf.subordinate_delay_off_master_usec.__str__()
    f_config['DEFAULT']['disable_streaming_indicator'] = dev_conf.disable_streaming_indicator.__str__()

    # Writing our configuration file to 'example.cfg'
    with open(f_config_name, 'w') as configfile:
        f_config.write(configfile)


    # dev_conf.color_resolution = convert_color_resolution(f_config['DEFAULT']['color_resolution'])
    # dev_conf.color_format = convert_color_format(f_config['DEFAULT']['color_format'])
    # dev_conf.depth_mode = convert_depth_mode(f_config['DEFAULT']['depth_mode'])
    # dev_conf.camera_fps = convert_camera_fps(f_config['DEFAULT']['camera_fps'])
    # dev_conf.synchronized_images_only = bool(f_config['DEFAULT']['synchronized_images_only'])
    # dev_conf.depth_delay_off_color_usec = int(f_config['DEFAULT']['depth_delay_off_color_usec'])
    # dev_conf.wired_sync_mode = convert_wired_sync_mode(f_config['DEFAULT']['wired_sync_mode'])
    # dev_conf.subordinate_delay_off_master_usec = int(f_config['DEFAULT']['subordinate_delay_off_master_usec'])
    # dev_conf.disable_streaming_indicator = bool(f_config['DEFAULT']['disable_streaming_indicator'])
    pass
    #return dev_conf


"""
Project:
Author:
Date:
Description:
 I don't know what is the source of this code, may be fixes of pyk4a

Use:
"""


import sys
import os
from path import Path

def _add_dll_directory(path: Path):
    from ctypes import c_wchar_p, windll  # type: ignore
    from ctypes.wintypes import DWORD

    AddDllDirectory = windll.kernel32.AddDllDirectory
    AddDllDirectory.restype = DWORD
    AddDllDirectory.argtypes = [c_wchar_p]
    AddDllDirectory(str(path))

def kinect():
    if sys.platform != "win32":
        return
    env_path = os.getenv("KINECT_LIBS", None)
    if env_path:
        candidate = Path(env_path)
        dll = candidate / "k4a.dll"
        if dll.exists():
            _add_dll_directory(candidate)
            return
    # autodetecting
    program_files = Path("C:\\Program Files\\")
    for dir in sorted(program_files.glob("Azure Kinect SDK v*"), reverse=True):
        candidate = dir / "sdk" / "windows-desktop" / "amd64" / "release" / "bin"
        dll = candidate / "k4a.dll"
        if dll.exists():
            _add_dll_directory(candidate)
            return



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: November 2021
Description:

Use:
from gui_frame_ext.about_window import AboutWindow

    def open_about_data(self):
        about_windows = AboutWindow(self)
        about_windows.grab_set()

"""

import os
import tkinter as tk


class AboutWindow2(tk.Toplevel):
    title_str = 'AK SM Recorder'
    version_number_str = '1.2.1'

    def __init__(self, parent):
        super().__init__(parent)
        self.geometry('300x480')
        self.title('About...')
        self.resizable(width=False, height=False)  # do not change the size
        self.attributes('-topmost', True)
        assets_path = os.path.dirname(os.path.abspath(__file__))
        img_path = os.path.join(assets_path, 'assets', 'icon_app.png')
        self.iconphoto(False, tk.PhotoImage(file=img_path))
        # ---------------------------

        about_label = tk.Label(self, text=self.title_str+' '+self.version_number_str)
        about_label.config(font=("Verdana", 12))
        about_label.pack(anchor=tk.CENTER)

        text_info = tk.Label(self)

        about_text_info = f' \n' \
                          f'Created by: Juan Carlos Miranda\n' \
                          f'Site: https://github.com/juancarlosmiranda\n' \
                          f'November 2021 \n' \
                          f' \n' \
                          f'PAgFRUIT project RTI2018-094222-B-I00\n' \
                          f'http://www.pagfruit.udl.cat/\n' \
                          f' \n' \
                          f'Research Group in AgroICT & Precision Agriculture\n' \
                          f'https://www.grap.udl.cat/\n'

        text_info['text'] = about_text_info
        text_info.pack(anchor=tk.CENTER)

        img_label = tk.Label(self)
        assets_path = os.path.dirname(os.path.abspath(__file__))
        img_path = os.path.join(assets_path, 'assets', 'logo_grap.png')
        img_label.image = tk.PhotoImage(file=img_path)
        img_label['image'] = img_label.image
        img_label.pack()

        button_close = tk.Button(self, text='Close', command=self.destroy)
        button_close.pack(expand=True)



"""
Project:
Author:
Date:
Description:
...

Use:
"""

import unittest
from test_job_thread_azure import TestJobThreadAzure
from test_job_thread_3D_azure import TestJobThread3DAzure

def camera_management_suite():
    """
        Gather all the tests for Azure camera
    """
    test_suite = unittest.TestSuite()
    #test_suite.addTest(unittest.makeSuite(TestDatasetConfig))
    test_suite.addTest(unittest.makeSuite(TestJobThreadAzure))
    test_suite.addTest(unittest.makeSuite(TestJobThread3DAzure))
    return test_suite


if __name__ == "__main__":
    runner = unittest.TextTestRunner()
    runner.run(camera_management_suite())


"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: August 2021
Description:
    This is an wraper to use Azure Kinect functions as a thread process
    This class is used when we need to launch process  from buttons

Use:

"""


import threading
import logging
import os
import helpers.helper_path as hp
# hp.kinect()
import pyk4a
import numpy as np
from pyk4a import Config, PyK4A, PyK4ARecord
from pyk4a import ColorResolution, ImageFormat, DepthMode, FPS
from datetime import datetime


class JobThreadAzure(threading.Thread):
    # Kinect device
    _device = None
    _capture = None
    device_config = None
    _f_path = None
    _mode_of_use = None

    # IR
    _infrared_show = None
    # Depth
    _depth_show = None
    # Depth
    _rgb_show = None

    def __init__(self, device_config=None, f_path_output=None):
        print('__INIT__')
        if device_config is None:
            logging.debug("Default conf loaded")
            # Load conf by default
            self.device_config = Config(
                color_resolution=ColorResolution.RES_720P,
                color_format=ImageFormat.COLOR_BGRA32,
                depth_mode=DepthMode.NFOV_UNBINNED,
                camera_fps=FPS.FPS_30,
                synchronized_images_only=True,
                depth_delay_off_color_usec=0,
                wired_sync_mode=pyk4a.WiredSyncMode.STANDALONE,
                subordinate_delay_off_master_usec=0,
                disable_streaming_indicator=False
            )
        else:
            logging.debug('External conf loaded')
            self.device_config = device_config

        if f_path_output is None:
            logging.debug("Default f_path_output loaded")
            BASE_DIR = os.path.abspath('.')
            self.f_path = os.path.join(BASE_DIR, 'recorded_video')
        else:
            self._f_path = f_path_output

        threading.Thread.__init__(self)
        self.shutdown_flag = threading.Event()

    def __del__(self):
        print('__DEL__')
        # logging.debug("__del__(self): - Finalize Azure Manager")
        # self.finalize_sensor()

    def initialize_sensor(self):
        logging.debug("Initialize_sensor()")
        self._device = PyK4A(self.device_config)
        self._device.start()

    def finalize_sensor(self):
        logging.debug("finalize_sensor()")
        self._device.stop()

    def get_file_name(self):
        print("Recording data")
        resolutionp = "_"
        resolutionp = self.device_config.color_resolution.name

        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_720P):
            resolutionp = "1080"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1080P):
            resolutionp = "1080"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1440P):
            resolutionp = "1440"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1536P):
            resolutionp = "1536"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_2160P):
            resolutionp = "2160P"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_3072P):
            resolutionp = "3072P"

        now = datetime.now()
        date_string = now.strftime("%d%m%Y%H%M%S")
        f_extension = ".mkv"
        f_name = resolutionp + "_" + date_string + f_extension
        f_path_name = os.path.join(self._f_path, f_name)
        logging.info(f"CREATING_FILE {f_path_name}")

        return f_path_name

    def run(self):
        # open sensor
        self.initialize_sensor()
        print("Recording data")
        f_path_name = self.get_file_name()
        print("Recording... Creating file.")
        record = PyK4ARecord(device=self._device, config=self.device_config, path=f_path_name)
        record.create()
        #####################
        # RECORD loop
        #####################
        try:
            print('RECORDING-STARTED #%s' % self.ident)
            logging.info(f'RECORDING-STARTED #{self.ident}')
            while not self.shutdown_flag.is_set():
                self._capture = self._device.get_capture()
                record.write_capture(self._capture)
        except Exception:
            print("An exception occurred")
        #####################
        record.flush()
        record.close()
        print(f"{record.captures_count} frames written.")
        # ... Clean shutdown code here ...
        print(f'RECORDING-STOPPED #{self.ident}')
        logging.info(f'RECORDING-STOPPED #{self.ident}')
        # close sensor sensor
        self.finalize_sensor()



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: August 2021
Description: Manager of Azure Kinect Sensor
    This is an examples adapted from:
    * https://github.com/UnaNancyOwen/AzureKinectSample/blob/master/sample/python/color.py
    * https://github.com/etiennedub/pyk4a/blob/master/example/

This is a first approach based on examples from the above urls adn modified from KA_manager1.py
This has simplified methods and it is used in
stand alone recording app.

  * Configure sensor
  * Take a capture with all cameras
  * Show each camera results
  * Show config data of one sensor
# todo: add descriptions

Use:

"""

import numpy as np
import cv2 as cv
import logging
import os
import pyk4a
from pyk4a import Config, ImageFormat, PyK4A, PyK4ARecord
from datetime import datetime


class KAManager2:
    # Kinect device
    my_device = None
    _capture = None
    device_config = None
    _f_path = None

    # IR
    _infrared_show = None
    # Depth
    _depth_show = None
    # Depth
    _rgb_show = None

    def __init__(self, device_config=None, f_path_output=None):
        if device_config is None:
            logging.debug("Default config loaded")
            # Load config by default
            self.device_config = Config(
                color_resolution=pyk4a.ColorResolution.RES_720P,
                color_format=pyk4a.ImageFormat.COLOR_BGRA32,
                depth_mode=pyk4a.DepthMode.NFOV_UNBINNED,
                camera_fps=pyk4a.FPS.FPS_30,
                synchronized_images_only=True,
                depth_delay_off_color_usec=0,
                wired_sync_mode=pyk4a.WiredSyncMode.STANDALONE,
                subordinate_delay_off_master_usec=0,
                disable_streaming_indicator=False
            )
            # todo: when we take a picture use BGRA32, WHEN WE RECORD DATA USE MPG
        else:
            logging.debug('External config loaded')
            self.device_config = device_config

        self._f_path = f_path_output
        # logging.debug("__init__(self): - Initialize loading Azure Manager")

    def __del__(self):
        logging.debug("__del__(self): - Finalize Azure Manager")
        self.finalize_sensor()

    def set_config_default(self):
        # todo: create configuration decoupled in object format
        device_config = Config()
        device_config.color_resolution = pyk4a.ColorResolution.OFF
        device_config.depth_mode = pyk4a.DepthMode.NFOV_UNBINNED
        device_config.synchronized_images_only = True
        self.device_config = device_config

    def initialize_sensor(self):
        logging.debug("Initialize_sensor()")
        self.my_device = PyK4A(self.device_config)
        self.my_device.start()

    def finalize_sensor(self):
        logging.debug("finalize_sensor()")
        self.my_device.stop()
        # self.my_device.close()
        pass

    def get_file_name(self):
        print("Recording data")
        resolution_p = "_"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_720P:
            resolution_p = "1080"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_1080P:
            resolution_p = "1080"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_1440P:
            resolution_p = "1440"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_1536P:
            resolution_p = "1536"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_2160P:
            resolution_p = "2160P"
        if self.device_config.color_resolution == pyk4a.ColorResolution.RES_3072P:
            resolution_p = "3072P"

        now = datetime.now()
        date_string = now.strftime("%d%m%Y%H%M%S")
        f_extension = ".mkv"
        f_name = resolution_p + "_" + date_string + f_extension
        f_path_name = os.path.join(self._f_path, f_name)
        logging.info(f"Recording... Creating file. {self._f_path}")

        return f_path_name

    def recording(self):
        self.initialize_sensor()
        print("Recording data")
        f_path_name = self.get_file_name()
        print("Recording... Creating file.")
        record = PyK4ARecord(device=self.my_device, config=self.device_config, path=f_path_name)
        record.create()
        #####################
        # RECORD loop
        #####################
        try:
            print("Recording... Press CTRL-C to stop recording.")
            while True:
                self._capture = self.my_device.get_capture()
                record.write_capture(self._capture)
        except KeyboardInterrupt:
            print("CTRL-C pressed. Exiting.")
        #####################
        record.flush()
        record.close()
        print(f"{record.captures_count} frames written.")
        self.finalize_sensor()

    def update_capture_frame(self):
        # Capture Frame
        self._capture = self.my_device.get_capture()
        if self._capture is None:
            raise IOError("failed get capture!")

    def draw(self):
        self.draw_infrared()
        self.draw_depth()
        self.draw_rgb()

    def draw_infrared(self):
        # Draw Infrared
        self._infrared_show = self._capture.ir

    def draw_depth(self):
        # Draw Infrared
        self._depth_show = self._capture.depth

    def draw_rgb(self):
        # Draw rgb
        self._rgb_show = self._capture.color

    def show(self):
        self.show_infrared()
        self.show_depth()
        self.show_rgb()

    def show_infrared(self):
        # Scaling Infrared
        # self._infrared_show = self._infrared_show * 0.5
        self._infrared_show = self._infrared_show.astype(np.uint8)
        cv.imshow("Infrared", self._infrared_show)

    def show_depth(self):
        # Scaling depth
        self._depth_show = self._depth_show.astype(np.uint8)
        cv.imshow("Depth", self._depth_show)

    def show_rgb(self):
        cv.imshow("RGB", self._rgb_show)

    def run_real_time(self):
        self.initialize_sensor()
        while True:
            self.update_capture_frame()
            self.draw()
            self.show()

            key = cv.waitKey(10)
            if key == ord('q'):
                cv.destroyAllWindows()
                break
        self.finalize_sensor()

    def show_a_capture(self):
        # todo: a problem with configuration mode in BGRA, may be variable assignment
        self.initialize_sensor()
        temporal = self.device_config.color_format
        self.device_config.color_format = ImageFormat.COLOR_BGRA32
        self.update_capture_frame()
        self.draw()
        self.show()
        key = cv.waitKey(10)
        self.device_config.color_format = temporal
        self.finalize_sensor()

    def get_a_capture(self):
        self.initialize_sensor()
        temporal = self.device_config.color_format
        self.device_config.color_format = ImageFormat.COLOR_BGRA32
        self.update_capture_frame()
        self.device_config.color_format = temporal
        self.finalize_sensor()
        return self._capture



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: April 2022
Description:

Use:
    python __main__.py
"""
import os
import sys
sys.path.append(os.path.join(os.path.abspath('.'), 'src'))
from os.path import expanduser

from gui_single_mode.gui_ak_single_config import GUIKASingleModeConfig
from gui_single_mode.gui_classes import GUIKASingleMode
from helpers.helper_filesystem import *

if __name__ == '__main__':
    user_path = expanduser("~")
    BASE_DIR = os.path.abspath('.')  # it gives the current location

    current_main_path_str = __file__
    package_path = os.path.join(os.path.dirname(os.path.normpath(current_main_path_str)), 'ak_sm_recorder')
    # ------------
    package_path_config_files = os.path.join(package_path, 'conf')
    package_path_log_files = os.path.join(package_path, 'log')

    path_conf_file = os.path.join(package_path_config_files, 'kinect_azure_settings.conf')
    ui_path_config_file = os.path.join(package_path_config_files, 'gui_ka_single_mode.conf')
    path_log_file = os.path.join(package_path_log_files, 'gui_ka_single_mode.log')  # todo: check if this is used

    # ------------
    root_folder = os.path.join(BASE_DIR, 'ak_sm_recorder')  #
    package_path_config_files = os.path.join(package_path, 'conf')
    path_user_config_files = os.path.join(root_folder, 'conf')
    path_user_output_folder = os.path.join(root_folder, 'recorded_video')

    print(f'user_path -> {user_path}')
    print('BASE_DIR->', BASE_DIR)
    print('current_main_path_str', current_main_path_str)
    print('package_path', package_path)
    print('path_gui_conf_file->', ui_path_config_file)
    print("package_path_config_files->", package_path_config_files)
    print('path_user_config_files->', path_user_config_files)

    # if directory doen't exist, then create
    if os.path.exists(root_folder):
        print('Directory exist!!!', root_folder)
    else:
        # creates hierarchy
        print('CREATING ', root_folder)
        print('CREATING ', path_user_config_files)
        print('CREATING ', path_user_output_folder)
        os.mkdir(root_folder)
        os.mkdir(path_user_config_files)
        os.mkdir(path_user_output_folder)

    if os.path.exists(path_user_config_files):
        print('Directory exist!!!', path_user_config_files)
    else:
        print('Directory doesn\'t exist!!!', path_user_config_files)
        print('Creating directory ', path_user_config_files)
        os.mkdir(path_user_config_files)
        copy_folder(package_path_config_files, path_user_config_files)

    # -------------------------
    if os.path.exists(path_user_output_folder):
        print('Directory exist!!!', path_user_output_folder)
    else:
        print('Directory doesn\'t exist!!!', path_user_output_folder)
        print('Creating directory ', path_user_output_folder)
        os.mkdir(path_user_output_folder)

    # -------------------------
    gui_config_obj = GUIKASingleModeConfig(ui_path_config_file)
    gui_config_obj.file_browser_input_folder = path_user_output_folder
    gui_config_obj.path_conf_file = path_conf_file
    # -------------------------
    app = GUIKASingleMode(gui_config_obj)
    app.mainloop()
    # -------------------------

    # INTERNATIONALIZATION
    # base file for translation is located in /base/
    # Use xgettext https://www.gnu.org/software/gettext/manual/html_node/xgettext-Invocation.html
    # xgettext -d gui_single_mode -o locale/gui_single_mode.pot gui_single_mode/gui_single_mode.py



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: April 2022
Description:
    This is an wrapper to use Azure Kinect functions 3D point clouds as a thread process.
    This class is used when we need to launch process  from buttons
Use:
"""

"""
Project: Kinect Azure management
Author: Juan Carlos Miranda
Date: April 2022
Description:
This is an wrapper to use Azure Kinect functions 3D point clouds as a thread process
This class is used when we need to launch process  from loops as
a remote client or buttons

Use:
"""

import threading
import logging
import os
import pyk4a
import numpy as np
from pyk4a import Config, PyK4A
from pyk4a import ColorResolution, ImageFormat, DepthMode, FPS
from datetime import datetime


class JobThread3DAzure(threading.Thread):
    # Kinect device
    _device = None
    _capture = None
    device_config = None
    _f_path = None
    _mode_of_use = None

    # IR
    _infrared_show = None
    # Depth
    _depth_show = None
    # Depth
    _rgb_show = None

    def __init__(self, device_config=None, f_path_output=None):
        print('__INIT__')
        if device_config is None:
            logging.debug("Default conf loaded")
            # Load conf by default
            self.device_config = Config(
                color_resolution=ColorResolution.RES_720P,
                color_format=ImageFormat.COLOR_BGRA32,
                depth_mode=DepthMode.NFOV_UNBINNED,
                camera_fps=FPS.FPS_30,
                synchronized_images_only=True,
                depth_delay_off_color_usec=0,
                wired_sync_mode=pyk4a.WiredSyncMode.STANDALONE,
                subordinate_delay_off_master_usec=0,
                disable_streaming_indicator=False
            )
        else:
            logging.debug('External conf loaded')
            self.device_config = device_config

        if f_path_output is None:
            logging.debug("Default f_path_output loaded")
            BASE_DIR = os.path.abspath('.')
            self.f_path = os.path.join(BASE_DIR, 'recorded_video')
        else:
            self._f_path = f_path_output

        threading.Thread.__init__(self)
        self.shutdown_flag = threading.Event()

    def __del__(self):
        print('__DEL__')
        # logging.debug("__del__(self): - Finalize Azure Manager")
        # self.finalize_sensor()

    def initialize_sensor(self):
        logging.debug("Initialize_sensor()")
        self._device = PyK4A(self.device_config)
        self._device.start()

    def finalize_sensor(self):
        logging.debug("finalize_sensor()")
        self._device.stop()

    def get_file_name(self):
        print("Recording data")
        resolutionp = "_"
        resolutionp = self.device_config.color_resolution.name

        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_720P):
            resolutionp = "1080"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1080P):
            resolutionp = "1080"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1440P):
            resolutionp = "1440"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_1536P):
            resolutionp = "1536"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_2160P):
            resolutionp = "2160P"
        if (self.device_config.color_resolution == pyk4a.ColorResolution.RES_3072P):
            resolutionp = "3072P"

        now = datetime.now()
        date_string = now.strftime("%d%m%Y%H%M%S")
        f_extension = ".xyz"
        f_name = resolutionp + "_" + date_string + f_extension
        f_path_name = os.path.join(self._f_path, f_name)
        logging.info(f"CREATING_FILE {f_path_name}")

        return f_path_name

    def run(self):
        """
        Adapted from example https://github.com/etiennedub/pyk4a/blob/master/example/viewer_point_cloud.py
        :return:
        """
        # open sensor
        self.initialize_sensor()
        print("Capturing 3D data")
        f_path_name = self.get_file_name()
        print("Capturing 3D data... Creating file.")
        # ------------------------------------------------
        # getters and setters directly get and set on device
        self._device.whitebalance = 4500
        assert self._device.whitebalance == 4500
        self._device.whitebalance = 4510
        assert self._device.whitebalance == 4510
        while True:
            capture = self._device.get_capture()
            if np.any(capture.depth) and np.any(capture.color):
                break
        while True:
            capture = self._device.get_capture()
            if np.any(capture.depth) and np.any(capture.color):
                break
        points_data = capture.depth_point_cloud.reshape((-1, 3))
        color_data = capture.transformed_color[..., (2, 1, 0)].reshape((-1, 3))
        point_cloud_3d = np.append(points_data, color_data, axis=1)
        np.savetxt(f_path_name, point_cloud_3d, delimiter=' ', fmt='%u')
        # ------------------------------------------------
        # print(f"{record.captures_count} frames written.")
        # ... Clean shutdown code here ...
        print(f'3D CAPTURING-STOPPED #{self.ident}')
        # logging.info(f'RECORDING-STOPPED #{self.ident}')
        # close sensor sensor
        self.finalize_sensor()



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: November 2021
Description:

Use:
from gui_frame_ext.about_window import AboutWindow

    def open_about_data(self):
        about_windows = AboutWindow(self)
        about_windows.grab_set()

"""

import os
import datetime
import time
import tkinter as tk
#from tkinter import *
from tkinter import ttk

import helpers.helper_load_config as hc
from pyk4a import Config, ColorResolution, ImageFormat, DepthMode
from pyk4a import FPS, WiredSyncMode

from gui_single_mode.about_window import AboutWindow2
from camera_classes.job_thread_azure import JobThreadAzure
from camera_classes.job_thread_3d_azure import JobThread3DAzure
from camera_classes.KA_manager2 import KAManager2


class GUIKASingleMode(tk.Tk):
    take_a_capture_flag = False

    FRAME_WIDTH = 100
    LABEL_WIDTH = 15
    ENTRY_WIDTH_PATH = 50
    BUTTON_WIDTH = 30

    TAB_NAME_1 = 'Camera'
    TAB_NAME_2 = 'Configuration'

    ui_config_obj = None
    path_video_output = None
    my_device_configuration = None
    # ----------------------------
    selected_depth = None
    depth_mode_list = None
    depth_mode_radio_0 = None
    depth_mode_radio_1 = None

    selected_color = None
    color_mode_list = None
    color_mode_radio = None

    selected_resolution = None
    resolution_mode_list = None
    resolution_mode_radio = None

    selected_framerate = None
    framerate_mode_list = None
    framerate_mode_radio = None

    # ----------------------------
    def __init__(self, ui_config_obj, master=None):
        super().__init__()
        # -----------------------
        self.geometry(ui_config_obj.geometry_main)
        self.title(ui_config_obj.app_title)
        self.resizable(width=False, height=False)
        self.attributes('-topmost', True)
        # -----------------------
        assets_path = os.path.dirname(os.path.abspath(__file__))
        img_path = os.path.join(assets_path, 'assets', 'icon_app.png')
        self.iconphoto(False, tk.PhotoImage(file=img_path))

        self.columnconfigure(0, weight=1)
        self.columnconfigure(1, weight=1)
        self.columnconfigure(2, weight=1)
        # ---------------
        self.ui_config_obj = ui_config_obj
        self.path_video_output = self.ui_config_obj.file_browser_input_folder
        self.initialize_device()
        # ---------------
        # calling methods to create components
        self.create_tabs()
        self.create_tab_page_1()
        self.create_tab_page_2()
        self.create_menu_bars()
        # ------

    def initialize_device(self):
        self.my_device_configuration = hc.load_config_from_file(self.ui_config_obj.path_conf_file)
        self.my_ka_manager = None
        #self.my_ka_manager = KAManager2(self.my_device_configuration, self.path_video_output)
        pass

    def create_tabs(self):
        self.tab_group = ttk.Notebook(self)
        self.tab_1 = tk.Frame(self.tab_group)
        self.tab_2 = tk.Frame(self.tab_group)
        self.tab_group.bind("<<NotebookTabChanged>>", self.on_tab_selected)
        self.tab_group.add(self.tab_1, text=self.TAB_NAME_1)
        self.tab_group.add(self.tab_2, text=self.TAB_NAME_2)
        self.tab_group.pack(expand=1, fill="both")
        pass

    def on_tab_selected(self, event):
        print('on_tab_selected')
        selected_tab = event.widget.select()
        tab_text = event.widget.tab(selected_tab, "text")
        print(tab_text)
        if tab_text == "Configurations":
            print('Make something with Configurations')

        if tab_text == "Camera":
            print('Make something with Camera')
        pass

    def draw_depth_mode_radiobuttons(self):
        self.depth_mode_list = (
            (DepthMode.NFOV_2X2BINNED.name, DepthMode.NFOV_2X2BINNED.name),
            (DepthMode.NFOV_UNBINNED.name, DepthMode.NFOV_UNBINNED.name),
            (DepthMode.WFOV_2X2BINNED.name, DepthMode.WFOV_2X2BINNED.name),
            (DepthMode.WFOV_UNBINNED.name, DepthMode.WFOV_UNBINNED.name)
        )

        depth_mode_option = self.depth_mode_list[0]
        self.depth_mode_radio_0 = ttk.Radiobutton(
            self.depth_mode_frame,
            text=depth_mode_option[0],
            value=depth_mode_option[1],
            variable=self.selected_depth,
            command=self.process_depth_radiobutton
        )
        self.depth_mode_radio_0.pack(fill='x', padx=5, pady=5)

        depth_mode_option = self.depth_mode_list[1]
        self.depth_mode_radio_1 = ttk.Radiobutton(
            self.depth_mode_frame,
            text=depth_mode_option[0],
            value=depth_mode_option[1],
            variable=self.selected_depth,
            command=self.process_depth_radiobutton
        )
        self.depth_mode_radio_1.pack(fill='x', padx=5, pady=5)

        depth_mode_option = self.depth_mode_list[2]
        self.depth_mode_radio_2 = ttk.Radiobutton(
            self.depth_mode_frame,
            text=depth_mode_option[0],
            value=depth_mode_option[1],
            variable=self.selected_depth,
            command=self.process_depth_radiobutton
        )
        self.depth_mode_radio_2.pack(fill='x', padx=5, pady=5)

        depth_mode_option = self.depth_mode_list[3]
        self.depth_mode_radio_3 = ttk.Radiobutton(
            self.depth_mode_frame,
            text=depth_mode_option[0],
            value=depth_mode_option[1],
            variable=self.selected_depth,
            command=self.process_depth_radiobutton
        )
        self.depth_mode_radio_3.pack(fill='x', padx=5, pady=5)

    def draw_color_mode_radiobuttons(self):
        self.color_mode_list = (
            (ImageFormat.COLOR_BGRA32.name, ImageFormat.COLOR_BGRA32.name),
            (ImageFormat.COLOR_MJPG.name, ImageFormat.COLOR_MJPG.name),
            (ImageFormat.COLOR_NV12.name, ImageFormat.COLOR_NV12.name),
            (ImageFormat.COLOR_YUY2.name, ImageFormat.COLOR_YUY2.name)
        )

        color_mode_option = self.color_mode_list[0]
        self.color_mode_radio_0 = ttk.Radiobutton(
            self.color_mode_frame,
            text=color_mode_option[0],
            value=color_mode_option[1],
            variable=self.selected_color,
            command=self.process_color_mode_radiobutton
        )
        self.color_mode_radio_0.pack(fill='x', padx=5, pady=5)

        color_mode_option = self.color_mode_list[1]
        self.color_mode_radio_1 = ttk.Radiobutton(
            self.color_mode_frame,
            text=color_mode_option[0],
            value=color_mode_option[1],
            variable=self.selected_color,
            command=self.process_color_mode_radiobutton
        )
        self.color_mode_radio_1.pack(fill='x', padx=5, pady=5)

        color_mode_option = self.color_mode_list[2]
        self.color_mode_radio_2 = ttk.Radiobutton(
            self.color_mode_frame,
            text=color_mode_option[0],
            value=color_mode_option[1],
            variable=self.selected_color,
            command=self.process_color_mode_radiobutton
        )
        self.color_mode_radio_2.pack(fill='x', padx=5, pady=5)

        color_mode_option = self.color_mode_list[3]
        self.color_mode_radio_3 = ttk.Radiobutton(
            self.color_mode_frame,
            text=color_mode_option[0],
            value=color_mode_option[1],
            variable=self.selected_color,
            command=self.process_color_mode_radiobutton
        )
        self.color_mode_radio_3.pack(fill='x', padx=5, pady=5)
        # --------------

    def draw_resolution_mode_radiobuttons(self):
        print('draw_resolution_mode_radiobutton()')
        # --------------
        self.resolution_mode_list = (
            (ColorResolution.RES_720P.name, ColorResolution.RES_720P.name),
            (ColorResolution.RES_1080P.name, ColorResolution.RES_1080P.name),
            (ColorResolution.RES_1440P.name, ColorResolution.RES_1440P.name),
            (ColorResolution.RES_2160P.name, ColorResolution.RES_2160P.name),
            (ColorResolution.RES_1536P.name, ColorResolution.RES_1536P.name),
            (ColorResolution.RES_3072P.name, ColorResolution.RES_3072P.name),
        )

        resolution_mode_option = self.resolution_mode_list[0]
        self.resolution_mode_radio_0 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_0.pack(fill='x', padx=5, pady=5)

        resolution_mode_option = self.resolution_mode_list[1]
        self.resolution_mode_radio_1 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_1.pack(fill='x', padx=5, pady=5)

        resolution_mode_option = self.resolution_mode_list[2]
        self.resolution_mode_radio_2 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_2.pack(fill='x', padx=5, pady=5)

        resolution_mode_option = self.resolution_mode_list[3]
        self.resolution_mode_radio_3 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_3.pack(fill='x', padx=5, pady=5)

        resolution_mode_option = self.resolution_mode_list[4]
        self.resolution_mode_radio_4 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_4.pack(fill='x', padx=5, pady=5)

        resolution_mode_option = self.resolution_mode_list[5]
        self.resolution_mode_radio_5 = ttk.Radiobutton(
            self.resolution_mode_frame,
            text=resolution_mode_option[0],
            value=resolution_mode_option[1],
            variable=self.selected_resolution,
            command=self.process_resolution_mode_radiobutton
        )
        self.resolution_mode_radio_5.pack(fill='x', padx=5, pady=5)

    def draw_framerate_mode_radiobuttons(self):
        print('draw_framerate_mode_radiobuttons()')
        # --------------
        self.framerate_mode_list = (
            (FPS.FPS_30.name, FPS.FPS_30.name),
            (FPS.FPS_15.name, FPS.FPS_15.name),
            (FPS.FPS_5.name, FPS.FPS_5.name)
        )

        framerate_mode_option = self.framerate_mode_list[0]
        self.framerate_mode_radio_0 = ttk.Radiobutton(
            self.framerate_mode_frame,
            text=framerate_mode_option[0],
            value=framerate_mode_option[1],
            variable=self.selected_framerate,
            command=self.process_framerate_mode_radiobutton
        )
        self.framerate_mode_radio_0.pack(fill='x', padx=5, pady=5)

        framerate_mode_option = self.framerate_mode_list[1]
        self.framerate_mode_radio_1 = ttk.Radiobutton(
            self.framerate_mode_frame,
            text=framerate_mode_option[0],
            value=framerate_mode_option[1],
            variable=self.selected_framerate,
            command=self.process_framerate_mode_radiobutton
        )
        self.framerate_mode_radio_1.pack(fill='x', padx=5, pady=5)

        framerate_mode_option = self.framerate_mode_list[2]
        self.framerate_mode_radio_2 = ttk.Radiobutton(
            self.framerate_mode_frame,
            text=framerate_mode_option[0],
            value=framerate_mode_option[1],
            variable=self.selected_framerate,
            command=self.process_framerate_mode_radiobutton
        )
        self.framerate_mode_radio_2.pack(fill='x', padx=5, pady=5)

    def draw_radiobuttons(self):
        self.selected_depth = tk.StringVar()  # todo: to improve this because is dirty code
        self.selected_depth.set(self.my_device_configuration.depth_mode.name)  # default asignment from config
        self.selected_color = tk.StringVar()
        self.selected_color.set(self.my_device_configuration.color_format.name)
        self.selected_resolution = tk.StringVar()
        self.selected_resolution.set(self.my_device_configuration.color_resolution.name)
        self.selected_framerate = tk.StringVar()
        self.selected_framerate.set(self.my_device_configuration.camera_fps.name)

        self.draw_depth_mode_radiobuttons()
        self.draw_color_mode_radiobuttons()
        self.draw_resolution_mode_radiobuttons()
        self.draw_framerate_mode_radiobuttons()

        self.process_depth_radiobutton()
        self.process_color_mode_radiobutton()
        self.process_resolution_mode_radiobutton()
        self.process_framerate_mode_radiobutton()

    def disable_high_resolution_mode(self):
        # ----------
        self.resolution_mode_radio_1.config(state='disabled')
        self.resolution_mode_radio_1.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_2.config(state='disabled')
        self.resolution_mode_radio_2.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_3.config(state='disabled')
        self.resolution_mode_radio_3.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_4.config(state='disabled')
        self.resolution_mode_radio_4.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_5.config(state='disabled')
        self.resolution_mode_radio_5.pack(fill='x', padx=5, pady=5)
        # ----------
        # select first options
        self.selected_resolution.set(ColorResolution.RES_720P.name)
        self.resolution_mode_radio_0.config(state='normal')
        self.resolution_mode_radio_0.config(variable=self.selected_resolution)
        # ----------

    def disable_high_framerate_mode(self):
        self.selected_framerate.set(FPS.FPS_15.name)
        self.framerate_mode_radio_0.config(state='disabled')
        self.framerate_mode_radio_1.config(state='normal')
        self.framerate_mode_radio_1.config(variable=self.selected_framerate)

    def enable_depth_mode_radiobuttons(self):
        self.depth_mode_radio_0.config(state='normal')
        self.depth_mode_radio_0.pack(fill='x', padx=5, pady=5)
        self.depth_mode_radio_1.config(state='normal')
        self.depth_mode_radio_1.pack(fill='x', padx=5, pady=5)
        self.depth_mode_radio_2.config(state='normal')
        self.depth_mode_radio_2.pack(fill='x', padx=5, pady=5)
        self.depth_mode_radio_3.config(state='normal')
        self.depth_mode_radio_3.pack(fill='x', padx=5, pady=5)

    def enable_color_mode_radiobuttons(self):
        self.color_mode_radio_0.config(state='normal')
        self.color_mode_radio_0.pack(fill='x', padx=5, pady=5)
        self.color_mode_radio_1.config(state='normal')
        self.color_mode_radio_1.pack(fill='x', padx=5, pady=5)
        self.color_mode_radio_2.config(state='normal')
        self.color_mode_radio_2.pack(fill='x', padx=5, pady=5)
        self.color_mode_radio_3.config(state='normal')
        self.color_mode_radio_3.pack(fill='x', padx=5, pady=5)

    def enable_resolution_mode_radiobuttons(self):
        self.resolution_mode_radio_0.config(state='normal')
        self.resolution_mode_radio_0.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_1.config(state='normal')
        self.resolution_mode_radio_1.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_2.config(state='normal')
        self.resolution_mode_radio_2.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_3.config(state='normal')
        self.resolution_mode_radio_3.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_4.config(state='normal')
        self.resolution_mode_radio_4.pack(fill='x', padx=5, pady=5)
        self.resolution_mode_radio_5.config(state='normal')
        self.resolution_mode_radio_5.pack(fill='x', padx=5, pady=5)

    def enable_frame_rate_radiobuttons(self):
        self.framerate_mode_radio_0.config(state='normal')
        self.framerate_mode_radio_0.pack(fill='x', padx=5, pady=5)
        self.framerate_mode_radio_1.config(state='normal')
        self.framerate_mode_radio_1.pack(fill='x', padx=5, pady=5)
        self.framerate_mode_radio_2.config(state='normal')
        self.framerate_mode_radio_2.pack(fill='x', padx=5, pady=5)

    def process_depth_radiobutton(self):
        print('process_depth_radiobutton(self):', self.selected_depth.get())
        if self.selected_depth.get() == DepthMode.NFOV_2X2BINNED.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()

        elif self.selected_depth.get() == DepthMode.NFOV_UNBINNED.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()

        elif self.selected_depth.get() == DepthMode.WFOV_2X2BINNED.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()

        elif self.selected_depth.get() == DepthMode.WFOV_UNBINNED.name:
            self.disable_high_framerate_mode()

        if self.selected_color.get() == ImageFormat.COLOR_NV12.name:
            self.disable_high_resolution_mode()

        if self.selected_color.get() == ImageFormat.COLOR_YUY2.name:
            self.disable_high_resolution_mode()

    def process_color_mode_radiobutton(self):
        print('process_color_mode_radiobutton(self):', self.selected_color.get())

        if self.selected_color.get() == ImageFormat.COLOR_BGRA32.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()

        elif self.selected_color.get() == ImageFormat.COLOR_MJPG.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()

            if self.selected_depth.get() == DepthMode.WFOV_2X2BINNED:
                self.disable_high_framerate_mode()

            elif self.selected_depth.get() == DepthMode.WFOV_UNBINNED.name:
                self.disable_high_framerate_mode()

        elif self.selected_color.get() == ImageFormat.COLOR_NV12.name:
            # ----------
            self.disable_high_resolution_mode()
            self.disable_high_framerate_mode()
            # ----------

        elif self.selected_color.get() == ImageFormat.COLOR_YUY2.name:
            self.enable_depth_mode_radiobuttons()
            self.enable_color_mode_radiobuttons()
            self.enable_resolution_mode_radiobuttons()
            self.enable_frame_rate_radiobuttons()
            # ----------
            self.disable_high_resolution_mode()

            if self.selected_depth.get() == DepthMode.WFOV_UNBINNED.name:
                self.disable_high_framerate_mode()
            # ----------

    def process_resolution_mode_radiobutton(self):
        print('process_resolution_mode_radiobutton(self):', self.selected_resolution.get())
        pass

    def process_framerate_mode_radiobutton(self):
        print('process_framerate_mode_radiobutton(self):', self.selected_framerate.get())
        pass

    def process_radiobuttons(self):
        print('PROCESS RADIOBUTTONS')
        print('selected_depth ->', self.selected_depth.get())
        print('selected_color ->', self.selected_color.get())
        print('selected_resolution ->', self.selected_resolution.get())
        print('selected_framerate ->', self.selected_framerate.get())
        print('update main configuration')
        print('save in file')
        self.my_device_configuration = Config(
            color_resolution=hc.convert_color_resolution(self.selected_resolution.get()),
            color_format=hc.convert_color_format(self.selected_color.get()),
            depth_mode=hc.convert_depth_mode(self.selected_depth.get()),
            camera_fps=hc.convert_camera_fps(self.selected_framerate.get()),
            synchronized_images_only=True,
            depth_delay_off_color_usec=0,
            wired_sync_mode=WiredSyncMode.STANDALONE,
            subordinate_delay_off_master_usec=0,
            disable_streaming_indicator=False
        )
        hc.save_config_in_file(self.ui_config_obj.path_conf_file, self.my_device_configuration)
        self.initialize_device()
        pass

    def create_tab_page_2(self):
        self.depth_mode_frame = tk.LabelFrame(self.tab_2, text="Depth options", width=self.FRAME_WIDTH)
        self.depth_mode_frame.grid(row=1, column=1, padx=5, pady=5)

        self.color_mode_frame = tk.LabelFrame(self.tab_2, text="Color options", width=self.FRAME_WIDTH)
        self.color_mode_frame.grid(row=1, column=2, padx=5, pady=5)

        self.resolution_mode_frame = tk.LabelFrame(self.tab_2, text="Resolution options", width=self.FRAME_WIDTH)
        self.resolution_mode_frame.grid(row=2, column=1, padx=5, pady=5)

        self.framerate_mode_frame = tk.LabelFrame(self.tab_2, text="Framerate options", width=self.FRAME_WIDTH)
        self.framerate_mode_frame.grid(row=2, column=2, padx=5, pady=5)

        self.start_config_button = tk.Button(self.tab_2, text='Save config', command=self.process_radiobuttons,
                                             width=self.BUTTON_WIDTH, )
        self.start_config_button.grid(row=3, column=1, columnspan=3, padx=5, ipady=5)
        self.draw_radiobuttons()

    def create_tab_page_1(self):
        self.camera_cmd_frame = tk.LabelFrame(self.tab_1, text="Commands", relief=tk.RIDGE,
                                              width=self.FRAME_WIDTH)  # todo: change frame
        self.camera_cmd_frame.grid(row=1, column=1, sticky=tk.W, padx=5, pady=5)
        # ---------------
        self.show_real_time_button = tk.Button(self.camera_cmd_frame, text='Show real time',
                                               command=self.show_real_time,
                                               width=self.BUTTON_WIDTH)
        self.show_real_time_button.grid(row=1, column=1, sticky=tk.W, ipadx=3, ipady=3)

        self.start_record_button = tk.Button(self.camera_cmd_frame, text='Start record', command=self.enable_record,
                                             width=self.BUTTON_WIDTH)
        self.start_record_button.grid(row=2, column=1, sticky=tk.W, ipadx=3, ipady=3)

        self.stop_record_button = tk.Button(self.camera_cmd_frame, text='Stop record', command=self.stop_record,
                                            width=self.BUTTON_WIDTH)
        self.stop_record_button.grid(row=3, column=1, sticky=tk.W, ipadx=3, ipady=3)

        self.take_capture_button = tk.Button(self.camera_cmd_frame, text='Take capture', command=self.take_capture,
                                             width=self.BUTTON_WIDTH)
        self.take_capture_button.grid(row=4, column=1, sticky=tk.W, ipadx=3, ipady=3)

        self.take_3d_capture_button = tk.Button(self.camera_cmd_frame, text='Take 3D point cloud capture', command=self.take_3d_capture, width=self.BUTTON_WIDTH)
        self.take_3d_capture_button.grid(row=5, column=1, sticky=tk.W, ipadx=3, ipady=3)


        ########################################################
        # MESSAGE FRAME
        self.message_frame = tk.LabelFrame(self.tab_1, text="Info", relief=tk.RIDGE, width=self.FRAME_WIDTH)
        self.message_frame.grid(row=3, column=1, sticky=tk.W, padx=5, pady=5)

        self.messages_label = tk.Label(self.message_frame, text='Messages:')
        self.messages_label.grid(row=1, column=0, sticky=tk.W, ipadx=3, ipady=3)

        self.messages_info = tk.Text(self.message_frame, width=28, height=5)
        self.messages_info.grid(row=2, column=0, sticky=tk.W)
        # ---------------
        self.quit_button = tk.Button(self.tab_1, text='Quit', command=self.quit_app, width=32)
        self.quit_button.grid(row=5, column=1, sticky=tk.W, padx=5, pady=5)
        # ---------------
        self.ui_buttons_enable_all()
        # ---------------

    def update_tasks(self):
        execution_time_update = datetime.datetime.utcnow()
        # print(f"We make a task HERE!! {execution_time_update}")
        self.after(2000, self.update_tasks)

    def create_menu_bars(self):
        self.menubar = tk.Menu(self)
        self.menu_help = tk.Menu(self.menubar, tearoff=False)  # delete dash lines
        self.menu_help.add_command(label='About...', command=self.not_implemented_yet)
        self.menubar.add_cascade(menu=self.menu_help, label='About', underline=0)
        self.config(menu=self.menubar)  # add menu to window

    def ui_buttons_enable_all(self):
        self.show_real_time_button["state"] = "active"
        self.start_record_button["state"] = "active"
        self.stop_record_button["state"] = "disabled"
        self.take_capture_button["state"] = "active"

    def ui_buttons_disable_all(self):
        self.show_real_time_button["state"] = "active"
        self.start_record_button["state"] = "active"
        self.stop_record_button["state"] = "disabled"
        self.take_capture_button["state"] = "disabled"

    def ui_buttons_enable_record(self):
        self.show_real_time_button["state"] = "disabled"
        self.start_record_button["state"] = "disabled"
        self.stop_record_button["state"] = "active"
        self.take_capture_button["state"] = "disabled"

    def ui_buttons_stop_record(self):
        self.show_real_time_button["state"] = "active"
        self.start_record_button["state"] = "active"
        self.stop_record_button["state"] = "disabled"
        self.take_capture_button["state"] = "active"

    def ui_buttons_take_capture(self):
        self.show_real_time_button["state"] = "disabled"
        self.show_real_time_button["state"] = "disabled"
        self.start_record_button["state"] = "disabled"
        self.stop_record_button["state"] = "active"
        print("Taking a capture!")
        self.job_ak_camera = JobThreadAzure(self.my_device_configuration, self.path_video_output)
        self.job_ak_camera.start()
        print('Waiting seconds and recording data--> ')
        time.sleep(4)  # todo: put this in config
        self.job_ak_camera.shutdown_flag.set()
        self.job_ak_camera.join()
        print('Video saved in --> ', self.path_video_output)
        self.ui_buttons_enable_all()
        pass


    def ui_buttons_3d_take_capture(self):
        self.show_real_time_button["state"] = "disabled"
        self.show_real_time_button["state"] = "disabled"
        self.start_record_button["state"] = "disabled"
        self.stop_record_button["state"] = "active"
        print("Taking a 3d cloud point capture!")
        self.job_ak_3d_camera = JobThread3DAzure(self.my_device_configuration, self.path_video_output)
        self.job_ak_3d_camera.start()
        print('Waiting seconds and recording data--> ')
        time.sleep(1)  # todo: put this in config
        self.job_ak_3d_camera.shutdown_flag.set()
        self.job_ak_3d_camera.join()
        print('3d point cloud saved in --> ', self.path_video_output)
        self.ui_buttons_enable_all()
        pass

    def not_implemented_yet(self):
        print("Not implemented yet!!!")
        about_windows = AboutWindow2(self)
        about_windows.grab_set()

    def show_real_time(self):
        print('show_real_time->')
        temporal_configuration = self.my_device_configuration
        temporal_configuration.color_format = ImageFormat.COLOR_BGRA32
        self.my_ka_manager = KAManager2(temporal_configuration, self.path_video_output)
        self.my_ka_manager.run_real_time()
        #self.my_ka_manager.show_a_capture()
        #self.my_ka_manager = None
        pass

    def enable_record(self):
        self.messages_info.insert("end", "Enable recording" + "\n")
        self.ui_buttons_enable_record()
        # self.my_device_configuration.color_format = ImageFormat.COLOR_MJPG
        self.job_ak_camera = JobThreadAzure(self.my_device_configuration, self.path_video_output)
        self.job_ak_camera.start()

    def stop_record(self):
        self.messages_info.insert("end", "Stop recording" + "\n")
        self.ui_buttons_stop_record()
        self.job_ak_camera.shutdown_flag.set()
        self.job_ak_camera.join()

    def take_capture(self):
        self.messages_info.insert("end", "Taking capture" + "\n")
        self.ui_buttons_take_capture()
        self.messages_info.insert("end", "A capture was launch!!" + "\n")
        time.sleep(5)
        self.messages_info.insert("end", "Capture time finished" + "\n")
        pass

    def take_3d_capture(self):
        self.messages_info.insert("end", "Taking 3D point cloud capture!" + "\n")
        self.ui_buttons_3d_take_capture()
        self.messages_info.insert("end", "A capture was launch!!" + "\n")
        time.sleep(5)
        self.messages_info.insert("end", "3D point cloud capture time finished" + "\n")
        pass


    def quit_app(self):
        # ---------------------------------------------
        # close token, close connection
        # ---------------------------------------------
        # self.my_ka_manager.__del__()
        self.quit
        self.destroy()
        # ---------------------------------------------



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: November 2021
Description:
    Manage configurations of user interface
Use:

    ui_path_config_file = os.path.join(BASE_DIR, 'conf', 'ui_frames_extractor.conf')
    ui_frame_extractor_config = GUIFrameExtractorConfig(ui_path_config_file)

"""


import os
from os.path import expanduser
import configparser


class GUIKASingleModeConfig:
    app_title = 'AK SM Recorder v1.2.1'
    width = 320
    height = 480
    geometry_about = '300x480'
    geometry_main = '550x800'
    file_extension_to_search = '.mkv'
    input_test_folder = ""
    user_path = expanduser("~")
    base_folder = os.path.abspath('')
    file_browser_input_folder = os.path.join(base_folder)
    input_dataset_folder = base_folder

    path_video_output = None  # todo: check this variable to put datafolder
    path_conf_file = None  # todo: check this variable to put datafolder

    def __init__(self, file_config_name=None):
        if file_config_name is not None:
            if os.path.isfile(file_config_name):
                self.f_config_name = file_config_name
                self.read_config()

    def read_config(self):
        """
        Read config from file ui_frames_extractor.conf
        :return:
        """
        f_config = configparser.ConfigParser()
        f_config.read(self.f_config_name)
        self.width = f_config['DEFAULT']['WIDTH']
        self.height = f_config['DEFAULT']['HEIGHT']
        self.geometry_about = f_config['DEFAULT']['geometry_about']
        self.geometry_main = f_config['DEFAULT']['geometry_main']
        self.file_extension_to_search = f_config['DEFAULT']['file_extension_to_search']
        self.base_folder = f_config['DEFAULT']['base_folder']
        self.file_browser_input_folder = f_config['DEFAULT']['file_browser_input_folder']
        self.input_dataset_folder = f_config['DEFAULT']['input_dataset_folder']



"""
Project: AK_SM_RECORDER Azure Kinect SM Recorder https://github.com/GRAP-UdL-AT/ak_sm_recorder

* PAgFRUIT http://www.pagfruit.udl.cat/en/
* GRAP http://www.grap.udl.cat/

Author: Juan Carlos Miranda. https://github.com/juancarlosmiranda
Date: April 2022
Description:

Use:
"""

import os
import shutil

def remove_files(folder_to_search, extension_file_to_search):
    # ---------------------------------------------------
    label_training_files_list = []
    for a_filename in os.listdir(folder_to_search):
        if a_filename.endswith(extension_file_to_search):
            os.remove(folder_to_search + a_filename)
    # ---------------------------------------------------

def copy_folder(folder_source, folder_target):
    for a_filename in os.listdir(folder_source):
        print('Copying files ', a_filename)
        target_file = a_filename
        print(os.path.join(folder_source, a_filename), os.path.join(folder_target, target_file))
        shutil.copyfile(os.path.join(folder_source, a_filename),
                        os.path.join(folder_target, target_file))
    pass



"""
Project:
Author:
Date:
Description:
    Taken from https://github.com/etiennedub/pyk4a/
Use:
"""
from typing import Optional, Tuple

import cv2
import numpy as np

import pyk4a

def colorize(
    image: np.ndarray,
    clipping_range: Tuple[Optional[int], Optional[int]] = (None, None),
    colormap: int = cv2.COLORMAP_HSV,
) -> np.ndarray:
    if clipping_range[0] or clipping_range[1]:
        img = image.clip(clipping_range[0], clipping_range[1])
    else:
        img = image.copy()
    img = cv2.normalize(img, None, 0, 255, cv2.NORM_MINMAX, dtype=cv2.CV_8U)
    img = cv2.applyColorMap(img, colormap)
    return img